"""This module contains exception classes from Pyramid Web Framework."""

class ConfigurationError(Exception):
    """ Raised when inappropriate input values are supplied to an API
    method of a :term:`Configurator`"""
